@@ -1,7 +1,7 @@
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Default, Debug)]
 pub(crate) struct GroupMask {
-    include: u16,
-    exclude: u16,
+    pub include: u16,
+    pub exclude: u16,
 }
 
 impl GroupMask {
@@ -11,14 +11,14 @@ impl GroupMask {
         Self { include, exclude }
     }
 
-    pub const fn new_include_group(arity: usize) -> Self {
+    pub const fn include(arity: usize) -> Self {
         Self {
             include: (1 << arity) - 1,
             exclude: 0,
         }
     }
 
-    pub const fn new_exclude_group(prev_arity: usize, arity: usize) -> Self {
+    pub const fn exclude(prev_arity: usize, arity: usize) -> Self {
         if prev_arity != 0 {
             let exclude_count = arity - prev_arity;
 
