@@ -1,10 +1,8 @@
-pub use self::families::*;
 pub use self::info::*;
 
-pub(crate) use self::group::*;
+pub(crate) use self::families::*;
 pub(crate) use self::mask::*;
 
 mod families;
-mod group;
 mod info;
 mod mask;
