@@ -1,10 +1,11 @@
-use crate::components::{BorrowStorages, Component, ComponentSet, ComponentStorages};
+use crate::components::{Component, ComponentSet, ComponentStorages};
 use crate::layout::Layout;
 use crate::resources::{Resource, ResourceStorage};
 use crate::storage::{ComponentStorage, Entity, EntityStorage};
 use crate::utils::{ChangeTicks, NonZeroTicks, Ticks};
 use crate::world::{BorrowWorld, NoSuchEntity, TickOverflow};
 use std::any::TypeId;
+use std::mem;
 use std::num::NonZeroU64;
 use std::sync::atomic::{AtomicU64, Ordering};
 
@@ -26,7 +27,7 @@ pub struct World {
     pub(crate) id: WorldId,
     pub(crate) tick: NonZeroTicks,
     pub(crate) entities: EntityStorage,
-    pub(crate) components: ComponentStorages,
+    pub(crate) storages: ComponentStorages,
     pub(crate) resources: ResourceStorage,
 }
 
@@ -36,7 +37,7 @@ impl Default for World {
             id: WorldId::new(),
             tick: NonZeroTicks::new(1).unwrap(),
             entities: Default::default(),
-            components: Default::default(),
+            storages: Default::default(),
             resources: Default::default(),
         }
     }
@@ -55,7 +56,12 @@ impl World {
     /// through all the entities to ararange their components, so it is best
     /// called right after creating the `World`.
     pub fn set_layout(&mut self, layout: &Layout) {
-        self.components.set_layout(layout, self.entities.as_ref());
+        let mut storages = mem::take(&mut self.storages).into_storages();
+
+        unsafe {
+            self.storages = ComponentStorages::new(layout, &mut storages);
+            self.storages.group_all_components(self.entities.as_ref());
+        }
     }
 
     /// Creates a component storage for `T` if one doesn't already exist.
@@ -63,12 +69,19 @@ impl World {
     where
         T: Component,
     {
-        self.components.register::<T>()
+        self.storages.register::<T>()
+    }
+
+    pub(crate) unsafe fn register_with<F>(&mut self, type_id: TypeId, storage_builder: F)
+    where
+        F: FnOnce() -> ComponentStorage,
+    {
+        self.storages.register_with(type_id, storage_builder);
     }
 
     /// Check if a component type is registered.
     pub fn is_registered(&self, component_type_id: &TypeId) -> bool {
-        self.components.is_registered(component_type_id)
+        self.storages.is_registered(component_type_id)
     }
 
     /// Creates an `Entity` with the given `components` and returns it.
@@ -90,7 +103,7 @@ impl World {
         entity
     }
 
-    /// Creates new `Entities` with the components produced by
+    /// Creates new entities with the components produced by
     /// `components_iter`. Returns the newly created entities as a slice.
     pub fn create_entities<C, I>(&mut self, components_iter: I) -> &[Entity]
     where
@@ -111,56 +124,55 @@ impl World {
         C: ComponentSet,
         I: IntoIterator<Item = C>,
     {
-        let initial_entity_count = self.entities.as_ref().len();
-
-        let families = {
-            let (mut storages, families) = C::Storages::borrow_with_families(&self.components);
-            let entities = &mut self.entities;
-
-            components_iter.into_iter().for_each(|components| {
-                let entity = entities.create();
-
-                unsafe {
-                    C::insert(&mut storages, entity, components, ticks);
-                }
-            });
-
-            families
-        };
-
-        let new_entities = &self.entities.as_ref()[initial_entity_count..];
-
-        for i in families.indexes() {
-            for &entity in new_entities {
-                unsafe {
-                    self.components.grouped.group_components(i, entity);
-                }
-            }
+        unsafe {
+            C::extend(
+                &mut self.storages,
+                &mut self.entities,
+                components_iter,
+                ticks,
+            )
         }
-
-        new_entities
     }
 
-    /// Removes `entity` and all of its `components` from the world.
-    /// Returns `true` if `entity` existed in the world before the call.
+    /// Removes `entity` and all of its components from the `World`.
+    /// Returns `true` if the `Entity` was successfully removed.
     pub fn destroy_entity(&mut self, entity: Entity) -> bool {
         if !self.entities.destroy(entity) {
             return false;
         }
 
-        for i in 0..self.components.grouped.group_family_count() {
-            unsafe {
-                self.components.grouped.ungroup_components(i, entity);
-            }
-        }
+        self.storages.ungroup_all_components(Some(&entity));
 
-        for storage in self.components.iter_storages_mut() {
+        for storage in self.storages.iter_mut() {
             storage.remove_and_drop(entity);
         }
 
         true
     }
 
+    /// Removes all entities (and their components) produced by the iterator.
+    /// Returns the number of entities successfully removed.
+    pub fn destroy_entities<'a, E>(&mut self, entities: E) -> usize
+    where
+        E: IntoIterator<Item = &'a Entity>,
+        E::IntoIter: Clone,
+    {
+        let entities = entities.into_iter();
+
+        self.storages.ungroup_all_components(entities.clone());
+
+        for storage in self.storages.iter_mut() {
+            entities.clone().for_each(|&entity| {
+                storage.remove_and_drop(entity);
+            });
+        }
+
+        entities
+            .into_iter()
+            .map(|&entity| self.entities.destroy(entity) as usize)
+            .sum()
+    }
+
     /// Appends the given `components` to `entity` if `entity` exists in the
     /// `World`.
     pub fn append_components<C>(
@@ -189,16 +201,8 @@ impl World {
             return Err(NoSuchEntity);
         }
 
-        let families = unsafe {
-            let (mut storages, families) = C::Storages::borrow_with_families(&self.components);
-            C::insert(&mut storages, entity, components, ticks);
-            families
-        };
-
-        for i in families.indexes() {
-            unsafe {
-                self.components.grouped.group_components(i, entity);
-            }
+        unsafe {
+            C::insert(&mut self.storages, entity, components, ticks);
         }
 
         Ok(())
@@ -210,22 +214,7 @@ impl World {
     where
         C: ComponentSet,
     {
-        if !self.contains_entity(entity) {
-            return None;
-        }
-
-        let families = C::Storages::families(&self.components);
-
-        for i in families.indexes() {
-            unsafe {
-                self.components.grouped.ungroup_components(i, entity);
-            }
-        }
-
-        unsafe {
-            let mut storages = C::Storages::borrow(&self.components);
-            C::remove(&mut storages, entity)
-        }
+        unsafe { C::remove(&mut self.storages, entity) }
     }
 
     /// Deletes a component set from `entity`. This is faster than removing
@@ -234,21 +223,8 @@ impl World {
     where
         C: ComponentSet,
     {
-        if !self.contains_entity(entity) {
-            return;
-        }
-
-        let families = C::Storages::families(&self.components);
-
-        for i in families.indexes() {
-            unsafe {
-                self.components.grouped.ungroup_components(i, entity);
-            }
-        }
-
         unsafe {
-            let mut storages = C::Storages::borrow(&self.components);
-            C::delete(&mut storages, entity);
+            C::delete(&mut self.storages, entity);
         }
     }
 
@@ -257,17 +233,15 @@ impl World {
         self.entities.contains(entity)
     }
 
-    pub fn borrow<'a, T>(&'a self) -> T::Item
-    where
-        T: BorrowWorld<'a>,
-    {
-        T::borrow(self, self.tick.get() - 1)
+    /// Returns all the `entities` in the world as a slice.
+    pub fn entities(&self) -> &[Entity] {
+        self.entities.as_ref()
     }
 
     /// Removes all entities and components in the world.
     pub fn clear_entities(&mut self) {
         self.entities.clear();
-        self.components.clear();
+        self.storages.clear();
     }
 
     /// Inserts a resource of type `T` into the `World` and returns the previous
@@ -276,7 +250,8 @@ impl World {
     where
         T: Resource,
     {
-        self.insert_resource_with_ticks(resource, ChangeTicks::just_added(self.tick.get()))
+        let ticks = ChangeTicks::just_added(self.tick.get());
+        self.insert_resource_with_ticks(resource, ticks)
     }
 
     /// Same as `insert_resource`, but the `ChangeTicks` are provided by the
@@ -297,7 +272,8 @@ impl World {
         self.resources.remove::<T>()
     }
 
-    /// Check if a resource exists.
+    /// Returns `true` if the `World` contains a resource with the given
+    /// `TypeId`.
     pub fn contains_resource(&self, resource_type_id: &TypeId) -> bool {
         self.resources.contains(resource_type_id)
     }
@@ -310,13 +286,26 @@ impl World {
     /// Removes all entities, components and resources from the `World`.
     pub fn clear(&mut self) {
         self.entities.clear();
-        self.components.clear();
+        self.storages.clear();
         self.resources.clear();
     }
 
-    /// Advances the current world tick. Should be called after each game
+    /// Borrows a component view or resource view from the `World`.
+    pub fn borrow<'a, T>(&'a self) -> T::Item
+    where
+        T: BorrowWorld<'a>,
+    {
+        T::borrow(self, self.tick.get() - 1)
+    }
+
+    /// Sets the world tick used for change detection.
+    pub fn set_tick(&mut self, tick: NonZeroTicks) {
+        self.tick = tick;
+    }
+
+    /// Advances the world tick. Should be called after each game
     /// update for proper change detection.
-    pub fn increment_ticks(&mut self) -> Result<(), TickOverflow> {
+    pub fn increment_tick(&mut self) -> Result<(), TickOverflow> {
         if self.tick.get() != Ticks::MAX {
             self.tick = NonZeroTicks::new(self.tick.get() + 1).unwrap();
             Ok(())
@@ -326,22 +315,13 @@ impl World {
         }
     }
 
-    /// Returns all the `entities` in the world as a slice.
-    pub fn entities(&self) -> &[Entity] {
-        self.entities.as_ref()
+    /// Returns the world tick used for change detection.
+    pub fn tick(&self) -> Ticks {
+        self.tick.get()
     }
 
     /// Returns the `WorldId` which uniquely identifies this `World`.
     pub fn id(&self) -> WorldId {
         self.id
     }
-
-    /// Returns the current world tick used for change detection.
-    pub fn tick(&self) -> Ticks {
-        self.tick.get()
-    }
-
-    pub(crate) unsafe fn register_storage(&mut self, component: TypeId, storage: ComponentStorage) {
-        self.components.register_storage(component, storage);
-    }
 }
